@@ -1,21 +1,33 @@
 
 setMethod(
   "dbWriteTable", "spark_connection",
-  function(conn, name, value, temporary = getOption("sparklyr.dbwritetable.temp", FALSE), append = FALSE, repartition = 0, serializer = NULL) {
-    found <- dbExistsTable(conn, name)
+  function(conn, name, value, temporary = getOption("sparklyr.dbwritetable.temp", FALSE), overwrite = FALSE, append = FALSE, repartition = 0, serializer = NULL) {
+    found <- dbExistsTable(conn, name) && !overwrite
     if (found) {
       stop("Table ", name, " already exists")
     }
 
+    if (append && overwrite) {
+      stop("append and overwrite parameters cannot both be setted to TRUE.")
+    }
+
     temp_name <- if (identical(temporary, FALSE)) random_string("sparklyr_tmp_") else name
 
+    if (identical(append, TRUE)) {
+        mode <- "append"
+    } else if (identical(overwrite, TRUE)) {
+        mode <- "overwrite"
+    } else {
+        mode <- NULL
+    }
+
     spark_data_copy(conn, value, temp_name, repartition, serializer = serializer)
 
     if (identical(temporary, FALSE)) {
       spark_write_table(
         tbl(conn, temp_name),
         name,
-        if (identical(append, TRUE)) "append" else NULL
+        mode
       )
     }
 
@@ -32,8 +44,12 @@ setMethod(
 )
 
 
-setMethod("dbListTables", "spark_connection", function(conn) {
-  df <- df_from_sql(conn, "SHOW TABLES")
+setMethod("dbListTables", "spark_connection", function(conn, database = NULL) {
+  query <- "SHOW TABLES"
+  if (!is.null(database)) {
+    query <- paste(query, "FROM", database, sep = " ")
+  }
+  df <- df_from_sql(conn, query)
 
   if (nrow(df) <= 0) {
     character(0)
