@@ -153,7 +153,7 @@ ml_decision_tree_classifier.tbl_spark <- function(x, formula = NULL, max_depth =
 
 validator_ml_decision_tree_classifier <- function(.args) {
   .args <- ml_validate_decision_tree_args(.args)
-  .args[["thresholds"]] <- cast_nullable_double_list(.args[["thresholds"]])
+  .args[["thresholds"]] <- cast_double_list(.args[["thresholds"]], allow_null = TRUE)
   .args[["impurity"]] <- cast_choice(.args[["impurity"]], c("gini", "entropy"))
   .args
 }
