@@ -8,21 +8,15 @@ arrow_write_record_batch <- function(df, spark_version_number = NULL) {
   }
 
   withr::with_envvar(arrow_env_vars, {
-    # New in arrow 0.17: takes a data.frame and returns a raw buffer with Arrow data
-    if ("write_to_raw" %in% ls(envir = asNamespace("arrow"))) {
-      # Fixed in 0.17: arrow doesn't hardcode a GMT timezone anymore
-      # so set the local timezone to any POSIXt columns that don't have one set
-      # https://github.com/sparklyr/sparklyr/issues/2439
-      df[] <- lapply(df, function(x) {
-        if (inherits(x, "POSIXt") && is.null(attr(x, "tzone"))) {
-          attr(x, "tzone") <- Sys.timezone()
-        }
-        x
-      })
-      arrow::write_to_raw(df, format = "stream")
-    } else {
-      arrow::write_arrow(arrow::record_batch(!!!df), raw())
-    }
+    # Set the local timezone to any POSIXt columns that don't have one set
+    # https://github.com/sparklyr/sparklyr/issues/2439
+    df[] <- lapply(df, function(x) {
+      if (inherits(x, "POSIXt") && is.null(attr(x, "tzone"))) {
+        attr(x, "tzone") <- Sys.timezone()
+      }
+      x
+    })
+    arrow::write_to_raw(df, format = "stream")
   })
 }
 
@@ -90,6 +84,7 @@ read_bin <- function(con, what, n, endian = NULL) {
   UseMethod("read_bin")
 }
 
+#' @export
 read_bin.default <- function(con, what, n, endian = NULL) {
   if (is.null(endian)) readBin(con, what, n) else readBin(con, what, n, endian = endian)
 }
@@ -139,10 +134,12 @@ read_bin.spark_connection <- function(con, what, n, endian = NULL) {
   read_bin_wait(con, what, n, endian)
 }
 
+#' @export
 read_bin.spark_worker_connection <- function(con, what, n, endian = NULL) {
   read_bin_wait(con, what, n, endian)
 }
 
+#' @export
 read_bin.livy_backend <- function(con, what, n, endian = NULL) {
   read_bin.default(con$rc, what, n, endian)
 }
@@ -251,7 +248,7 @@ readDate <- function(con) {
   if (is.na(n)) {
     as.Date(NA)
   } else {
-    d <- as.Date(n, origin = "1970-01-01", tz = "UTC")
+    d <- as.Date(n, origin = "1970-01-01")
     if (getOption("sparklyr.collect.datechars", FALSE)) {
       as.character(d)
     } else {
@@ -266,7 +263,7 @@ readTime <- function(con, n = 1) {
   } else {
     t <- readDouble(con, n)
 
-    r <- as.POSIXct(t, origin = "1970-01-01", tz = "UTC")
+    r <- as.POSIXct(t, origin = "1970-01-01")
     if (getOption("sparklyr.collect.datechars", FALSE)) {
       as.character(r)
     } else {
@@ -615,8 +612,7 @@ core_invoke_synced <- function(sc) {
 
   if (length(returnStatus) == 0 || returnStatus != 0) {
     FALSE
-  }
-  else {
+  } else {
     object <- readObject(sc)
     identical(object, echo_id)
   }
@@ -729,14 +725,12 @@ core_invoke_method_impl <- function(sc, static, noreply, object, method, return_
       ), {
         if (nzchar(msg)) {
           core_handle_known_errors(sc, msg)
-
-          stop(msg, call. = FALSE)
         } else {
           # read the spark log
           msg <- core_read_spark_log_error(sc)
-          stop(msg, call. = FALSE)
         }
       })
+      spark_error(msg)
     }
 
     result_object <- readObject(sc)
@@ -748,6 +742,7 @@ core_invoke_method_impl <- function(sc, static, noreply, object, method, return_
   attach_connection(result_object, sc)
 }
 
+#' @export
 jobj_subclass.shell_backend <- function(con) {
   "shell_jobj"
 }
@@ -756,6 +751,7 @@ jobj_subclass.spark_connection <- function(con) {
   "shell_jobj"
 }
 
+#' @export
 jobj_subclass.spark_worker_connection <- function(con) {
   "shell_jobj"
 }
@@ -769,8 +765,7 @@ core_handle_known_errors <- function(sc, msg) {
       "Failed to retrieve localhost, please validate that the hostname is correctly mapped. ",
       "Consider running `hostname` and adding that entry to your `/etc/hosts` file."
     )
-  }
-  else if (grepl("check worker logs for details", msg, ignore.case = TRUE) &&
+  } else if (grepl("check worker logs for details", msg, ignore.case = TRUE) &&
     spark_master_is_local(sc$master)) {
     abort_shell(
       "sparklyr worker rscript failure, check worker logs for details",
@@ -796,6 +791,81 @@ core_read_spark_log_error <- function(sc) {
   })
   msg
 }
+
+
+spark_error <- function(message) {
+  option_name <- "sparklyr.simple.errors"
+  simple_errors <- unlist(options(option_name))
+
+  if (is.null(simple_errors)) {
+    use_simple <- FALSE
+  } else {
+    use_simple <- simple_errors
+  }
+
+  if (use_simple) {
+    stop(message, call. = FALSE)
+  }
+
+  split_message <- unlist(strsplit(message, "\n\t"))
+
+  msg_l <- "\u001B]8;;"
+  msg_r <- "\u001B\\"
+  msg_fn <- "sparklyr::spark_last_error()"
+
+  term <- Sys.getenv("TERM")
+  color_terms <- c("xterm-color", "xterm-256color", "screen", "screen-256color")
+
+  check_rstudio <- try(RStudio.Version(), silent = TRUE)
+
+  in_rstudio <- TRUE
+  if (inherits(check_rstudio, "try-error")) {
+    in_rstudio <- FALSE
+  }
+  if (term %in% color_terms) {
+    if (in_rstudio) {
+      scheme <- "ide:run"
+    } else {
+      scheme <- "x-r-run"
+    }
+
+    msg_fun <- paste0(
+      msg_l, scheme, ":", msg_fn, msg_r, "`", msg_fn, "`", msg_l, msg_r
+    )
+  } else {
+    msg_fun <- paste0("`", msg_fn, "`")
+  }
+
+  last_err <- paste0(
+    "Run ", msg_fun, " to see the full Spark error (multiple lines)"
+  )
+
+  option_msg <- paste(
+    "To use the previous style of error message",
+    "set `options(\"sparklyr.simple.errors\" = TRUE)`"
+  )
+
+  msg <- c(split_message[[1]], "", last_err, option_msg)
+
+  #genv_set_last_error(message)
+
+  rlang::abort(
+    message = msg,
+    use_cli_format = TRUE,
+    call = NULL
+  )
+}
+
+#' Surfaces the last error from Spark captured by internal `spark_error` function
+#' @export
+spark_last_error <- function() {
+  last_error <- genv_get_last_error()
+  if (!is.null(last_error)) {
+    rlang::inform(last_error)
+  } else {
+    rlang::inform("No error found")
+  }
+}
 #' Retrieve a Spark JVM Object Reference
 #'
 #' This S3 generic is used for accessing the underlying Java Virtual Machine
@@ -1220,6 +1290,7 @@ worker_config_serialize <- function(config) {
     if (isTRUE(config$arrow)) "TRUE" else "FALSE",
     if (isTRUE(config$fetch_result_as_sdf)) "TRUE" else "FALSE",
     if (isTRUE(config$single_binary_column)) "TRUE" else "FALSE",
+    if (isTRUE(config$spark_read)) "TRUE" else "FALSE",
     config$spark_version,
     sep = ";"
   )
@@ -1227,7 +1298,6 @@ worker_config_serialize <- function(config) {
 
 worker_config_deserialize <- function(raw) {
   parts <- strsplit(raw, ";")[[1]]
-
   list(
     debug = as.logical(parts[[1]]),
     sparklyr.gateway.port = as.integer(parts[[2]]),
@@ -1237,7 +1307,8 @@ worker_config_deserialize <- function(raw) {
     arrow = as.logical(parts[[6]]),
     fetch_result_as_sdf = as.logical(parts[[7]]),
     single_binary_column = as.logical(parts[[8]]),
-    spark_version = parts[[9]]
+    spark_read = as.logical(parts[[9]]),
+    spark_version = parts[[10]]
   )
 }
 # nocov start
@@ -1253,9 +1324,7 @@ spark_worker_context <- function(sc) {
       connection = sc
     )
   )
-
   worker_log("retrieved worker context")
-
   context
 }
 
@@ -1334,15 +1403,11 @@ spark_worker_execute_closure <- function(
   result <- do.call(closure, closure_args)
   worker_log("computed closure")
 
-  as_factors <- getOption("stringsAsFactors")
-  on.exit(options(stringsAsFactors = as_factors))
-  options(stringsAsFactors = FALSE)
-
   if (identical(fetch_result_as_sdf, FALSE)) {
     serialize_impl <- spark_worker_get_serializer(sc)
     result <- lapply(result, function(x) serialize_impl(x, NULL))
     class(result) <- c("spark_apply_binary_result", class(result))
-    result <- tibble::tibble(spark_apply_binary_result = result)
+    result <- dplyr::tibble(spark_apply_binary_result = result)
   }
 
   if (!"data.frame" %in% class(result)) {
@@ -1367,7 +1432,7 @@ spark_worker_maybe_serialize_list_cols_as_json <- function(config, result) {
     config$spark_version >= "2.4.0" &&
     any(sapply(result, is.list))) {
     result <- do.call(
-      tibble::tibble,
+      dplyr::tibble,
       lapply(
         result,
         function(x) {
@@ -1607,9 +1672,18 @@ spark_worker_apply <- function(sc, config) {
   length <- worker_invoke(context, "getSourceArrayLength")
   worker_log("found ", length, " rows")
 
-  groups <- worker_invoke(context, if (grouped) "getSourceArrayGroupedSeq" else "getSourceArraySeq")
-  worker_log("retrieved ", length(groups), " rows")
+  is_read <- config$spark_read %||% FALSE
 
+  groups_func <- if(grouped) {
+    "getSourceArrayGroupedSeq"
+  } else if (is_read && config$spark_version >= "4") {
+    "getSourceArraySeq2"
+  } else {
+    "getSourceArraySeq"
+  }
+  groups <- worker_invoke(context, groups_func)
+
+  worker_log("retrieved ", length(groups), " rows")
   deserialize_impl <- spark_worker_get_deserializer(sc)
 
   closureRaw <- worker_invoke(context, "getClosure")
@@ -1643,10 +1717,12 @@ spark_worker_apply <- function(sc, config) {
   for (group_entry in groups) {
     # serialized groups are wrapped over single lists
     data <- group_entry[[1]]
-
+    if(config$spark_version >= "4" && !grouped && !is_read) {
+      data <- lapply(data, unlist, recursive = FALSE)
+    }
     df <- (
       if (config$single_binary_column) {
-        tibble::tibble(encoded = lapply(data, function(x) x[[1]]))
+        dplyr::tibble(encoded = lapply(data, function(x) x[[1]]))
       } else {
         bind_rows <- core_get_package_function("dplyr", "bind_rows")
         as_tibble <- core_get_package_function("tibble", "as_tibble")
@@ -1658,7 +1734,7 @@ spark_worker_apply <- function(sc, config) {
             )
           )
         } else {
-          warning("dplyr::bind_rows or tibble::as_tibble is unavailable, ",
+          warning("dplyr::bind_rows or dplyr::as_tibble is unavailable, ",
                   "falling back to rbind implementation in base R. ",
                   "Inputs with list column(s) will not work.")
 
@@ -1845,7 +1921,7 @@ spark_worker_connect <- function(
 
 # nocov end
 # nocov start
-
+#' @export
 connection_is_open.spark_worker_connection <- function(sc) {
   bothOpen <- FALSE
   if (!identical(sc, NULL)) {
@@ -1879,6 +1955,7 @@ worker_invoke <- function(jobj, method, ...) {
   UseMethod("worker_invoke")
 }
 
+#' @export
 worker_invoke.shell_jobj <- function(jobj, method, ...) {
   worker_invoke_method(worker_connection(jobj), FALSE, jobj, method, ...)
 }
@@ -1953,7 +2030,6 @@ spark_worker_main <- function(
                               backendPort = 8880,
                               configRaw = NULL) {
   spark_worker_hooks()
-
   tryCatch(
     {
       worker_log_session(sessionId)
