@@ -7,11 +7,12 @@ import java.text.SimpleDateFormat
 import java.util.Calendar
 import java.util.TimeZone
 
-import scala.collection.JavaConverters._
-import scala.collection.convert.Wrappers.SeqWrapper
+import scala.jdk.CollectionConverters._
 import scala.collection.mutable.WrappedArray
 import scala.Option
 
+import org.apache.spark.sql.types.{StructType, DataType}
+
 object Serializer {
   private[this] val kMsPerDay: Long = 24 * 60 * 60 * 1000
 
@@ -376,11 +377,14 @@ class Serializer(tracker: JVMObjectTracker) {
       val value = (
         if (obj.isInstanceOf[WrappedArray[_]]) {
           obj.asInstanceOf[WrappedArray[_]].toArray
+        } else if (obj.isInstanceOf[StructType]) {
+          obj
+        } else if (obj.isInstanceOf[Array[_]]) {
+          obj
+        } else if (obj.isInstanceOf[Seq[_]]) {
+          obj.asInstanceOf[Seq[_]].mkString(",")
         } else {
-          obj match {
-            case s @ SeqWrapper(_) => s.toArray
-            case _ => obj
-          }
+          obj
         }
       )
 
