@@ -1,12 +1,9 @@
 package sparklyr
 
 import java.io.{File, FileWriter}
-
-import org.apache.spark._
 import org.apache.spark.broadcast.Broadcast
-import org.apache.spark.sql.Row
+import org.apache.spark.sql.{Row, SparkSession}
 import org.apache.spark.sql.types.StructType
-
 import scala.util.Random
 
 class WorkerApply(
@@ -27,31 +24,27 @@ class WorkerApply(
   partitionIndexProvider: () => Int,
   serializer: Broadcast[Array[Byte]],
   deserializer: Broadcast[Array[Byte]]
-) extends java.io.Serializable {
+) extends Serializable {
 
-  private[this] var exception: Option[Exception] = None
-  private[this] var backendPort: Int = 0
+  @transient private[this] var exception: Option[Exception] = None
+  @transient private[this] var backendPort: Int = 0
 
   def workerSourceFile(rscript: Rscript, sessionId: Int): String = {
     val source = Sources.sources
-
-    val tempFile: File = new File(
-      rscript.getScratchDir() + File.separator + "sparkworker_" + sessionId.toString + ".R")
-    val outStream: FileWriter = new FileWriter(tempFile)
+    val tempFile = new File(s"${rscript.getScratchDir}${File.separator}sparkworker_$sessionId.R")
+    val outStream = new FileWriter(tempFile)
     outStream.write(source)
     outStream.flush()
-
-    tempFile.getAbsolutePath()
+    outStream.close()
+    tempFile.getAbsolutePath
   }
 
   def apply(iterator: Iterator[Row]): Iterator[Row] = {
-
-    val sessionId: Int = Random.nextInt(10000)
+    val sessionId = Random.nextInt(10000)
     val logger = new Logger("Worker", sessionId)
-    val lock: AnyRef = new Object()
+    val lock = new Object()
 
-    // No point in starting up R process to not process anything
-    if (!iterator.hasNext) return Array[Row]().iterator
+    if (!iterator.hasNext) return Iterator.empty
 
     val workerContext = new WorkerContext(
       iterator,
@@ -73,96 +66,79 @@ class WorkerApply(
 
     val tracker = new JVMObjectTracker()
     val contextId = tracker.put(workerContext)
-    logger.log("is tracking worker context under " + contextId)
+    logger.log(s"is tracking worker context under $contextId")
 
     logger.log("initializing backend")
-    val backend: Backend = new Backend()
+    val backend = new Backend()
     backend.setTracker(tracker)
-
-    /*
-     * initialize backend as worker and service, since exceptions and
-     * terminating the r session should not shutdown the process
-     */
     backend.setType(
       true,   /* isService */
       false,  /* isRemote */
       true,   /* isWorker */
       false   /* isBatch */
     )
+    backend.setHostContext(contextId)
+    backend.init(port, sessionId, connectionTimeout)
+    backendPort = backend.getPort
 
-    backend.setHostContext(
-      contextId
-    )
-
-    backend.init(
-      port,
-      sessionId,
-      connectionTimeout
-    )
-
-    backendPort = backend.getPort()
-
-    new Thread("starting backend thread") {
+    val backendThread = new Thread("backend thread") {
       override def run(): Unit = {
         try {
           logger.log("starting backend")
-
           backend.run()
         } catch {
           case e: Exception =>
             logger.logError("failed while running backend: ", e)
             exception = Some(e)
             lock.synchronized {
-              lock.notify
+              lock.notify()
             }
         }
       }
-    }.start()
+    }
+    backendThread.setDaemon(true)
+    backendThread.start()
 
-    new Thread("starting rscript thread") {
+    val rscriptThread = new Thread("rscript thread") {
       override def run(): Unit = {
         try {
-          logger.log("is starting rscript")
-
+          logger.log("starting rscript")
           val rscript = new Rscript(logger)
-          val sourceFilePath: String = workerSourceFile(rscript, sessionId)
-
+          val sourceFilePath = workerSourceFile(rscript, sessionId)
           rscript.init(
-            List(
-              sessionId.toString,
-              backendPort.toString,
-              config
-            ),
+            List(sessionId.toString, backendPort.toString, config),
             sourceFilePath,
             customEnv,
             options
           )
-
           lock.synchronized {
-            lock.notify
+            lock.notify()
           }
         } catch {
           case e: Exception =>
             logger.logError("failed to run rscript: ", e)
             exception = Some(e)
             lock.synchronized {
-              lock.notify
+              lock.notify()
             }
         }
       }
-    }.start()
+    }
+    rscriptThread.setDaemon(true)
+    rscriptThread.start()
 
-    logger.log("is waiting using lock for RScript to complete")
+    logger.log("waiting for RScript to complete")
     lock.synchronized {
       lock.wait()
     }
-    logger.log("completed wait using lock for RScript")
+    logger.log("RScript completed")
 
-    if (exception.isDefined) {
-      throw exception.get
+    exception match {
+      case Some(x) => throw x
+      case _    => println("")
     }
 
-    logger.log("is returning RDD iterator with " + workerContext.getResultArray().length + " rows")
-    return workerContext.getResultArray().iterator
+    logger.log(s"returning RDD iterator with ${workerContext.getResultArray.length} rows")
+    workerContext.getResultArray.iterator
   }
 }
