@@ -4,7 +4,7 @@ import org.apache.spark._
 import org.apache.spark.rdd.RDD
 import org.apache.spark.sql._
 import org.apache.spark.sql.types.StructType
-import scala.collection.JavaConversions._
+import scala.jdk.CollectionConverters._
 
 class WorkerContext(
   iterator: Iterator[Row],
@@ -59,7 +59,11 @@ class WorkerContext(
     getSourceArray.length
   }
 
-  def getSourceArraySeq(): Array[Seq[Any]] = {
+  def getSourceArraySeq(): Array[Row] = {
+    getSourceArray
+  }
+
+  def getSourceArraySeq2(): Array[Seq[Any]] = {
     getSourceArray.map(x => x.toSeq)
   }
 
